
//ABB Measurement & Analytics
//3400 rue Pierre-Ardoin
//Québec (Qc) G1P 0B2
//Canada

// 
// Copyright (c) 2013 ABB, Ltd.  All rights reserved.
// 
// 
// ABB is supplying this software for use solely and
// exclusively on ABB products.
// 
// The software is owned by ABB, and is protected under
// applicable copyright laws.                                                

/***************************************************************************/
/*                                                                      
    DD Revision 1.1  18/02/2016
    Raghu Ram
*/
/***************************************************************************/
/* File-Name: 1aa201_Standard.ddl                                            */
/***************************************************************************/

MANUFACTURER ABB, DEVICE_TYPE _LLT100, DEVICE_REVISION 1, DD_REVISION 2   

#include "macros.h"
#include "methods.h"
#include "1aa201_strings.h"
#include "1aa201_enums.h"
#include "1aa201_vars.dd"
#include "1aa201_lintable.dd"
#include "1aa201_commands.dd"
#include "1aa201_Methods.dd"
#include "1aa201_Menus.dd"
#include "1aa201_EnhancedMenus.dd"


IMPORT STANDARD _TABLES, DEVICE_REVISION 20, DD_REVISION 4
{
    HART_DEVICE_TABLES;
    BURST_TABLES;

    REDEFINITIONS
    {
        VARIABLE device_type
        {
            /* 
             * The type must be redefined if the Device Type is not listed
             * in Common Tables DD
             */
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0xf9f9 with your Expanded Device Type code and appropriate label
                { 0x1aa2,  "LLT100" } 
            }
        }
        VARIABLE manufacturer_id
        {
            /*
             * The type must be redefined if the Manufacturer code is not listed
             * in the Common Tables DD
             */
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0x00f9 with your Manufacturer code and appropriate label
                { 0x001a,  [ABB] } 
            }
        }
        VARIABLE private_label_distributor
        {
            /*
             * The type must be redefined if the Private Label code is not listed
             * in the Common Tables DD
             */
            REDEFINE TYPE ENUMERATED (2)
            {
                // Replace the 0x00f9 with your manufactures id and appropriate label
                { 0x001a,  [ABB] } 
            }
        }
        VARIABLE loop_current_mode
        {
            REDEFINE TYPE ENUMERATED
            {
                { 0, "Fixed"    },
                { 1, "4-20 mA"  }
            }
        }
        VARIABLE write_protect
        {
            REDEFINE LABEL "HW Write Protection";
        }
    }
}

/*
 * Universal Commands
*/

IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 7, DD_REVISION 2
{
    EVERYTHING;  
	REDEFINITIONS
	 {
        COMMAND read_device_variables_and_status
        {
            REDEFINE TRANSACTION
            {
                REQUEST
                {
                    device_variable_code_1 (INFO,INDEX),
                    device_variable_code_2 (INFO,INDEX),
                    device_variable_code_3 (INFO,INDEX),
                    device_variable_code_4 (INFO,INDEX)
        
                }
                REPLY
                {
                    response_code, device_status,
                    extended_fld_device_status,
            
                    device_variable_code_1 (INFO,INDEX),
                    deviceVariables[device_variable_code_1].CLASSIFICATION,
                    deviceVariables[device_variable_code_1].DIGITAL_UNITS,
                    deviceVariables[device_variable_code_1].DIGITAL_VALUE,
                    deviceVariables[device_variable_code_1].DATA_QUALITY <0xC0>,
                    deviceVariables[device_variable_code_1].LIMIT_STATUS <0x30>,
                    deviceVariables[device_variable_code_1].DEVICE_FAMILY_STATUS <0x0F>,
            
                    device_variable_code_2 (INFO,INDEX),
                    deviceVariables[device_variable_code_2].CLASSIFICATION,
                    deviceVariables[device_variable_code_2].DIGITAL_UNITS,
                    deviceVariables[device_variable_code_2].DIGITAL_VALUE,
                    deviceVariables[device_variable_code_2].DATA_QUALITY <0xC0>,
                    deviceVariables[device_variable_code_2].LIMIT_STATUS <0x30>,
                    deviceVariables[device_variable_code_2].DEVICE_FAMILY_STATUS <0x0F>,
            
                    device_variable_code_3 (INFO,INDEX),
                    deviceVariables[device_variable_code_3].CLASSIFICATION,
                    deviceVariables[device_variable_code_3].DIGITAL_UNITS,
                    deviceVariables[device_variable_code_3].DIGITAL_VALUE,
                    deviceVariables[device_variable_code_3].DATA_QUALITY <0xC0>,
                    deviceVariables[device_variable_code_3].LIMIT_STATUS <0x30>,
                    deviceVariables[device_variable_code_3].DEVICE_FAMILY_STATUS <0x0F>,
            
                    device_variable_code_4 (INFO,INDEX),
                    deviceVariables[device_variable_code_4].CLASSIFICATION,
                    deviceVariables[device_variable_code_4].DIGITAL_UNITS,
                    deviceVariables[device_variable_code_4].DIGITAL_VALUE,
                    deviceVariables[device_variable_code_4].DATA_QUALITY <0xC0>,
                    deviceVariables[device_variable_code_4].LIMIT_STATUS <0x30>,
                    deviceVariables[device_variable_code_4].DEVICE_FAMILY_STATUS <0x0F>
					time_stamp
                }
            }
        }
        VARIABLE response_preambles
        {
            REDEFINE HANDLING READ & WRITE;
        }
         /*
         * Must delete the data that the device
         * does not support in Command 48
         */
        DELETE VARIABLE device_specific_status_24;

        // Remove the command 48 status bytes that are not supported by the device
        // See also command 48 related items in the common practice imports.
       
        VARIABLE device_specific_status_0
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				//device_specific_status_0_ENUM
               /* { 0x01, "Level sensor failure", 
                        "Possible Cause: Level sensor failure. \n\nSuggested Action: Contact ABB Level Support at http://www.abb.com/level", 
                         HARDWARE },*/
				{ 0x01, "Undefined", IGNORE_IN_HANDHELD},
			    { 0x02, "Undefined", IGNORE_IN_HANDHELD},
                { 0x04, "Write protect failure", 
                        "Possible Cause: Write protect condition failed\n\nSuggested Action:Contact ABB if persists", 
                         HARDWARE },
                { 0x08, "Safety function flow error", 
                        "Possible Cause:Firmware error\n\nSuggested Action:Reset device,Contact ABB if persists", 
                         HARDWARE },
                { 0x10, "Level sensor out of limits", 
                        "Possible Cause: Measured distance too high/low\n\nSuggested Action: Check for level sensor failure.", 
                         PROCESS },
                { 0x20, "Undefined", IGNORE_IN_HANDHELD},
                { 0x40, "Electronics MCU failure", 
                        "Possible Cause: Self test failure. \n\nSuggested Action: Contact ABB Support", 
                         HARDWARE },
                { 0x80, "Undefined", IGNORE_IN_HANDHELD }

            }
		}
        
        VARIABLE device_specific_status_1
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
                { 0x01, "Undefined", IGNORE_IN_HANDHELD },
                { 0x02, "Undefined", IGNORE_IN_HANDHELD },
                { 0x04, "Level Sensor Out of Range", 
                        "Possible Cause:4-20mA Configuration not matching process.\n\nSuggested Action:Review configuration", 
                         PROCESS }, // It was Primary Value Out of Range LLT                        
                { 0x08, "Undefined", IGNORE_IN_HANDHELD },
                { 0x10, "Undefined", IGNORE_IN_HANDHELD },
                { 0x20, "Undefined", IGNORE_IN_HANDHELD },
                { 0x40, "Undefined", IGNORE_IN_HANDHELD },
                { 0x80, "Undefined", IGNORE_IN_HANDHELD }
            }
		}
        VARIABLE device_specific_status_2
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{ 0x01, "Sensor board failure",HARDWARE  },
                { 0x02, "Undefined", IGNORE_IN_HANDHELD},
                { 0x04, "Undefined", IGNORE_IN_HANDHELD },
                { 0x08, "Device reset required", 
                        "Possible Cause:Action require reset\n\nSuggested Action:Cycle power for changes to take effect", 
                         HARDWARE },
                { 0x10, "Sensor communication error", 
                        "Possible Cause:Electronic failure\n\nSuggested Action:Reset device. Contact ABB if persists", 
                         HARDWARE },
                { 0x20, "Undefined", IGNORE_IN_HANDHELD },
                { 0x40, "Undefined", IGNORE_IN_HANDHELD },
                { 0x80, "Electronics NV memory failure", 
                        "Possible Cause:Firmware error\n\nSuggested Action:Contact ABB Service", 
                         HARDWARE }
			}
		}
        VARIABLE device_specific_status_3
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{0x01, "Undefined", IGNORE_IN_HANDHELD },
                {0x02, "Undefined", IGNORE_IN_HANDHELD },
                {0x04, "Undefined", IGNORE_IN_HANDHELD },
                {0x08, "Undefined", IGNORE_IN_HANDHELD },
                {0x10, "Undefined", IGNORE_IN_HANDHELD },
                {0x20, "Undefined", IGNORE_IN_HANDHELD },
                {0x40, "HMI validation test failure",
                       "Possible Cause:Firmware error\n\nSuggested Action:Reset device.if persists replace HMI.",
                        MODE },
                {0x80, "Front-end read-back amplitude too high", 
                       "Possible Cause:Windows obstructed or laser blocked\n\nSuggested Action:Check process and device. Contact ABB if persists.", 
                        HARDWARE }
			}
		}
		VARIABLE device_specific_status_4
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{0x01, "Electronics insufficient input voltage", 
                       "Possible Cause:Insufficient voltage to operate device\n\nSuggested Action: Check power supply.", 
                        HARDWARE },
                {0x02, "Ambient temperature out of range", 
                       "Possible Cause:Ambient temperature out of range\n\nSuggested Action:Readings may be invalid", 
                        PROCESS },
                {0x04, "Undefined", IGNORE_IN_HANDHELD },
                {0x08, "Sensor not calibrated", 
                       "Possible Cause:Device was not calibrated in production\n\nSuggested Action:Contact ABB Service", 
                        PROCESS },
                {0x10, "Data Simulation warning ", 
                       "Possible Cause:Device in simulation mode\n\nSuggested Action:One or more process values are in SIM mode", 
                        MODE },
                {0x20, "Alarm in Simulation  Mode ", 
                       "Possible Cause:Device in simulation mode\n\nSuggested Action:One or more alarms are in simulation mode", 
                        MODE },
                {0x40, "Current output in fixed mode", 
                       "Possible Cause: Current output set to fixed mode. \n\nSuggested Action:Process should be in manual control mode", 
                        MODE },
                {0x80, "Primary current uncertain", 
                       "Possible Cause:Electronic failure\n\nSuggested Action:Contact ABB support if condition persists", 
                        HARDWARE }
			}
		}
		VARIABLE device_specific_status_5
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
                {0x01, "Electronics ROM failure", 
                       "Possible Cause:Electronic failure\n\nSuggested Action:Contact ABB support if condition persists", 
                        HARDWARE },
                {0x02, "Electronics RAM failure", 
                       "Possible Cause:Electronic failure\n\nSuggested Action:Contact ABB support if condition persists", 
                        HARDWARE },
                {0x04, "Target lost delay expired", 
                       "Possible Cause:Dust or vapour in process.\n\nSuggested Action:Check Process conditions", 
                        HARDWARE },
                {0x08, "Primary current saturated", 
                       "Possible Cause:4-20mA Range too low\n\nSuggested Action:Contact ABB support if condition persists", 
                        HARDWARE },
                {0x10, "Current output read back failed", 
                       "Possible Cause: Current output readback failed. \n\nSuggested Action:Contact ABB support if condition persists", 
                        HARDWARE },
                {0x20, "Process media warning", 
                       "Possible Cause:Invalid process conditions not matching configuration\n\nSuggested Action:Monitor Process Conditions", 
                        PROCESS },
                {0x40, "Process media alarm", 
                       "Possible Cause:Invalid process conditions not matching configuration\n\nSuggested Action: Invalid    process conditions not matching configuration", 
                        PROCESS },
                {0x80,  "Algorithm configuration invalid", "Possible Cause:Invalid configuration parameters. \n\nSuggested Action:Adjust Config.  Parameters",IGNORE_IN_HANDHELD}
			}
		}

        VARIABLE device_specific_status_14
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{ 0x01, "Undefined", IGNORE_IN_HANDHELD },
                { 0x02, "Undefined", IGNORE_IN_HANDHELD },
                { 0x04, "Undefined", IGNORE_IN_HANDHELD },
                { 0x08, "Undefined", IGNORE_IN_HANDHELD },
                { 0x10, "Undefined", IGNORE_IN_HANDHELD },
                { 0x20, "Undefined", IGNORE_IN_HANDHELD },
                { 0x40, "Undefined", IGNORE_IN_HANDHELD },
                { 0x80, "Undefined", IGNORE_IN_HANDHELD }
            }
		}
                
        VARIABLE device_specific_status_15
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{0x01, "None", IGNORE_IN_HANDHELD },
                {0x02, "Check function", MODE },
                {0x04, "Off specification", PROCESS },
                {0x08, "Maintenance", HARDWARE },
                {0x10, "Failure", HARDWARE },
                {0x20, "Undefined", IGNORE_IN_HANDHELD },
                {0x40, "Undefined", IGNORE_IN_HANDHELD },
                {0x80, "Undefined", IGNORE_IN_HANDHELD }
			}
		}
        VARIABLE device_specific_status_16
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{0x01, "Electronics status", HARDWARE },
                {0x02, "Sensor status", HARDWARE },
                {0x04, "Configuration status", PROCESS },
                {0x08, "Operating conditions", PROCESS },
                {0x10, "Process operating conditions", PROCESS },
                {0x20, "Undefined", IGNORE_IN_HANDHELD },
                {0x40, "Undefined", IGNORE_IN_HANDHELD },
                {0x80, "Undefined", IGNORE_IN_HANDHELD }
			}
		}

        VARIABLE device_specific_status_17
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{ 0x01, "Undefined", IGNORE_IN_HANDHELD},
                { 0x02, "Undefined", IGNORE_IN_HANDHELD},
                { 0x04, "Write protect failure", 
                        "Possible Cause: Write protect condition failed\n\nSuggested Action:Contact ABB if persists", 
                         IGNORE_IN_HANDHELD },
                { 0x08, "Safety function flow error", 
                        "Possible Cause:Firmware error\n\nSuggested Action:Reset device. Contact ABB if persists", 
                         IGNORE_IN_HANDHELD },
                { 0x10, "Level sensor out of limits", 
                        "Possible Cause: Measured distance too high/low\n\nSuggested Action: Check for level sensor failure.", 
                         IGNORE_IN_HANDHELD },
                { 0x20, "Undefined", IGNORE_IN_HANDHELD},
                { 0x40, "Electronic MCU failure", 
                        "Possible Cause: Self test failure. \n\nSuggested Action: Contact ABB Support", 
                         IGNORE_IN_HANDHELD },
                { 0x80, "Undefined", IGNORE_IN_HANDHELD }
            }
		}
        VARIABLE device_specific_status_18
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
                { 0x01, "Undefined", IGNORE_IN_HANDHELD },
                { 0x02, "Undefined", IGNORE_IN_HANDHELD },
                { 0x04, "Level Sensor Out of Range", 
                        "Possible Cause:4-20mA Configuration not matching process.\n\nSuggested Action:Review configuration", 
                         IGNORE_IN_HANDHELD },   // It was Primary Value Out of Range LLT                        
                { 0x08, "Undefined", IGNORE_IN_HANDHELD },
                { 0x10, "Undefined", IGNORE_IN_HANDHELD },
                { 0x20, "Undefined", IGNORE_IN_HANDHELD },
                { 0x40, "Undefined", IGNORE_IN_HANDHELD },
                { 0x80, "Undefined", IGNORE_IN_HANDHELD }
			}
		}
        VARIABLE device_specific_status_19
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
                { 0x01, "Sensor board failure", IGNORE_IN_HANDHELD },
                { 0x02, "Undefined", IGNORE_IN_HANDHELD },
                { 0x04, "Undefined", IGNORE_IN_HANDHELD },
                { 0x08, "Device Reset Required",IGNORE_IN_HANDHELD },
                { 0x10, "Sensor communication error",IGNORE_IN_HANDHELD },
                { 0x20, "Undefined", IGNORE_IN_HANDHELD },
                { 0x40, "Undefined", IGNORE_IN_HANDHELD },
                { 0x80, "Electronics NV memory failure",IGNORE_IN_HANDHELD }
			}
		}
		VARIABLE device_specific_status_20
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
                {0x01, "Undefined", IGNORE_IN_HANDHELD },
                {0x02, "Undefined", IGNORE_IN_HANDHELD },
                {0x04, "Undefined", IGNORE_IN_HANDHELD },
                {0x08, "Undefined", IGNORE_IN_HANDHELD },
                {0x10, "Undefined", IGNORE_IN_HANDHELD },
                {0x20, "Undefined", IGNORE_IN_HANDHELD },
                {0x40, "HMI validation test failure",IGNORE_IN_HANDHELD },
                {0x80, "Front-end read-back amplitude too high",IGNORE_IN_HANDHELD}
			}
		}
		VARIABLE device_specific_status_21
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{0x01, "Electronics insufficient input voltage",IGNORE_IN_HANDHELD },
                {0x02, "Ambient temperature out of range",IGNORE_IN_HANDHELD },
                {0x04, "Undefined", IGNORE_IN_HANDHELD  },
                {0x08, "Sensor not calibrated", 
                        PROCESS },
                {0x10, "Data Simulation warning ",IGNORE_IN_HANDHELD},
                {0x20, "Alarm in simulation Mode",IGNORE_IN_HANDHELD },
                {0x40, "Current output in fixed mode",IGNORE_IN_HANDHELD }
                {0x80, "Primary current uncertain",IGNORE_IN_HANDHELD }
			}
		}
		VARIABLE device_specific_status_22
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{0x01, "Electronics ROM failure",IGNORE_IN_HANDHELD },
                {0x02, "Electronics RAM failure",IGNORE_IN_HANDHELD },
                {0x04, "Target lost delay expired",IGNORE_IN_HANDHELD },
                {0x08, "Primary current saturated",IGNORE_IN_HANDHELD },
                {0x10, "Current output read back failed",IGNORE_IN_HANDHELD },
                {0x20, "Process media warning",IGNORE_IN_HANDHELD },
                {0x40, "Process media alarm",IGNORE_IN_HANDHELD },
                {0x80, "Algorithm configuration invalid", IGNORE_IN_HANDHELD }
			}
		}
        VARIABLE device_specific_status_23
		{
			REDEFINE TYPE BIT_ENUMERATED
			{
				{0x01, "Diagnostics Simulation", MODE },
                {0x02, "Undefined", IGNORE_IN_HANDHELD },
                {0x04, "Undefined", IGNORE_IN_HANDHELD },
                {0x08, "Undefined", IGNORE_IN_HANDHELD },
                {0x10, "Undefined", IGNORE_IN_HANDHELD },
                {0x20, "Undefined", IGNORE_IN_HANDHELD },
                {0x40, "Undefined", IGNORE_IN_HANDHELD },
                {0x80, "Undefined", IGNORE_IN_HANDHELD }
			}
		}
        COMMAND read_additional_device_status
        {
            REDEFINE TRANSACTION 0
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0, device_specific_status_1, device_specific_status_2,
                    device_specific_status_3, device_specific_status_4, device_specific_status_5,

                    extended_fld_device_status, 0x00,

                    standardized_status_0, standardized_status_1, analog_channel_saturated1,
                    standardized_status_2, standardized_status_3, analog_channel_fixed1,

                    device_specific_status_14, device_specific_status_15, device_specific_status_16,
                    device_specific_status_17, device_specific_status_18, device_specific_status_19,
                    device_specific_status_20, device_specific_status_21, device_specific_status_22,
                    device_specific_status_23
                }
            }
            REDEFINE TRANSACTION 1
            {
                REQUEST
                {
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),

                    extended_fld_device_status (INFO), 0x00,

                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO), analog_channel_fixed1 (INFO),

                    device_specific_status_14(INFO), device_specific_status_15(INFO), device_specific_status_16(INFO),
                    device_specific_status_17(INFO), device_specific_status_18(INFO), device_specific_status_19(INFO),
                    device_specific_status_20(INFO), device_specific_status_21(INFO), device_specific_status_22(INFO),
                    device_specific_status_23 (INFO)
                }
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),

                    extended_fld_device_status (INFO), 0x00,

                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO), analog_channel_fixed1 (INFO),
                    
                    device_specific_status_14 (INFO), device_specific_status_15 (INFO), device_specific_status_16 (INFO),
                    device_specific_status_17 (INFO), device_specific_status_18 (INFO), device_specific_status_19 (INFO),
                    device_specific_status_20 (INFO), device_specific_status_21 (INFO), device_specific_status_22 (INFO),
                    device_specific_status_23 (INFO) 
                }
            }
        }
    }
   
}

IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
{
    EVERYTHING;
    REDEFINITIONS
    {
        /*
        VARIABLE loop_alarm_code    
        {
            REDEFINE HANDLING READ & WRITE;
            TYPE ENUMERATED
            {
                DELETE 239;
                DELETE 251;
            }
        } */
        /* 
         * Most devices use the same data item for range units (command 15) and the sensor units.
         * when this is true the following should be added.  if your device uses a different variable
         * for range units you should reference it here 
         */
        COLLECTION OF VARIABLE scaling
        {
            MEMBERS
            {
                ADD RANGE_UNITS, deviceVariables[primary_variable_code].DIGITAL_UNITS, [range_units], [range_units_help];
            } 
        }
        REDEFINE UNIT scaling_units_relation
        {
            deviceVariables[primary_variable_code].DIGITAL_UNITS:
            upperRange_value,
            lowerRange_value
        }
        COLLECTION OF COLLECTION primary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE,        deviceVariables[primary_variable_code];
            }
        }
        COLLECTION OF COLLECTION secondary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE,        deviceVariables[secondary_variable_code];
            }
        }
        COLLECTION OF COLLECTION tertiary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE,        deviceVariables[tertiary_variable_code];
            }
        }
        COLLECTION OF COLLECTION quaternary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE,        deviceVariables[quaternary_variable_code];
            }
        }
        VARIABLE loopCurrent
    	{
            REDEFINE HANDLING READ & WRITE;            
    	}

        ARRAY OF COLLECTION dynamic_variables
        {
            ELEMENTS
            {
                REDEFINE QUATERNARY,     quaternary_variable,    ABB_QV;
            }
        }
    }
}

/*
 * Common Practice Commands
 */

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 9, DD_REVISION 2
{
    VARIABLE device_variable_code;
    VARIABLE burst_command_number;
    VARIABLE primary_variable_code;
    VARIABLE secondary_variable_code;
    VARIABLE tertiary_variable_code;
    VARIABLE quaternary_variable_code;
    VARIABLE device_variable_trim_code;

    //Commands
    COMMAND read_device_variables;                  /* command 33 */
	COMMAND write_pv_damping_value; 				/* command 34 */
	COMMAND write_pv_range_values;					/* Command 35 */
    COMMAND set_pv_upper_range_value;          	    /* Command 36 */
    COMMAND set_pv_lower_range_value;         	    /* Command 37 */
   	COMMAND enter_exit_fixed_pv_current_mode;		/* Command 40 */
	COMMAND perform_self_test;			            /* Command 41 */
    COMMAND perform_device_reset;                   /* Command 42 */
	COMMAND write_pv_units;                         /* Command 44 */
	COMMAND trim_pv_current_dac_zero;				/* Command 45 */
	COMMAND trim_pv_current_dac_gain;				/* Command 46 */
	COMMAND read_device_variable_assignments;		/* command 50 */
	COMMAND write_device_variable_assignments;		/* command 51 */
    COMMAND set_device_variable_zero;               /* command 52 */
    COMMAND write_device_variable_units;            /* command 53 */
    COMMAND read_device_variable_information;       /* command 54 */
    COMMAND write_number_of_response_preambles;     /* command 59 */ // Removed in latest FW LLT Again re added
    COMMAND lock_device;                            /* command 71 */
    COMMAND read_lock_device_state;                 /* command 76 */
    COMMAND write_device_variable;                  /* Command 79 */
    COMMAND reset_device_variable_trim;             /* Command 83 */

    //Methods
    METHOD applied_rerange;
   	METHOD transmitter_loop_test;
	METHOD transmitter_dac_trim;
	METHOD leave_fixed_current_mode;
    METHOD lock_unlock_device;
    METHOD device_self_test;
    METHOD device_master_reset;
    METHOD return_to_normal;
    METHOD warning_message;
    METHOD device_variable_zero_trim;
    METHOD device_variable_trim;
    METHOD trim_warning;
    METHOD device_variable_trim_reset;
    METHOD fixed_analog_output;
  
   REDEFINITIONS
    {

        VARIABLE primary_variable_code
        {
            REDEFINE TYPE INDEX primaryvariables; 
        }
        VARIABLE secondary_variable_code
        {
            REDEFINE TYPE INDEX secondaryvariables; 
        }
        VARIABLE tertiary_variable_code
        {
            REDEFINE TYPE INDEX tertiaryvariables; 
        }
        VARIABLE quaternary_variable_code
        {
            REDEFINE TYPE INDEX quaternaryvariables; 
        }
        VARIABLE device_variable_trim_code
        {
            REDEFINE TYPE INDEX trimDeviceVariables; 
        }
      



        /*COMMAND perform_self_test  
        {
            RESPONSE_CODES
            {
               ADD 7,  MODE_ERROR,       [in_write_protect_mode];
            }
        }*/
    }
}

